create or replace library pypl_lib as '/opt/pypl/pypl.so';
/
create or replace package PYPL is

  -- Author  : Jan Holst Jensen, jan@biochemfusion.com.
  -- Created : 2014-02-24 21:41:55
  -- Purpose : Simple Python integration.

  MAX_PLSQL_VARCHAR2_LEN constant integer := 32000;
  -- PL/SQL won't allow me to declare the following using the constant above but it was the intention.
  subtype pls_largest_varchar2 is varchar2(32000);
  subtype pls_err_msg_varchar2 is varchar2(4000);

  -- Encode string so it can be safely passed to Python without risking code injection.
  function encoded_string(s varchar2) return varchar2;

  -- Reset Python runtime. Warning: Causes a slight memory leak every time it is used.
  -- The memory leak is a well-documented Python issue.
  procedure reset;

  -- Run a script and return the string value of a named Python variable.
  function run_script(script_text in varchar2, result_varname in varchar2) return varchar2;

end PYPL;
/
create or replace package body PYPL is

  RETURN_CODE_SUCCESS constant pls_integer := 0;

  -- External library functions.
  function lib_latest_err_msg
    return string
  is
    language c library pypl_lib name "latest_err_msg"
    parameters (return string);

  function lib_reset
    return pls_integer
  is
    language c library pypl_lib name "reset"
    parameters (return int);

  function lib_run_script(
    script_text in string, result_varname in string, result out string
  )
    return pls_integer
  is
    language c library pypl_lib name "run_script"
    parameters (
      script_text string, result_varname string, result string, result maxlen,
      return int
    );

  -- Internal helper functions.

  -- Test return value of function, raise appropriate error message
  -- if return value indicates failure.
  procedure check_return(return_value in pls_integer)
  is
    err_msg pls_err_msg_varchar2;
  begin
    if return_value <> RETURN_CODE_SUCCESS then
      err_msg := lib_latest_err_msg();
      raise_application_error(-20000, 'pypl: ' || err_msg);
    end if;
  end;

  -- Public PL/SQL functions.
  function encoded_string(s varchar2) return varchar2
  is
  begin
    return replace(s, '"', '\"');
  end;

  procedure reset
  is
  begin
    check_return(lib_reset);
  end;

  function run_script(script_text in varchar2, result_varname in varchar2) return varchar2
  is
    result pls_largest_varchar2;
  begin
    if result_varname is null then
      check_return(lib_run_script(script_text, ' ', result));
    else
      check_return(lib_run_script(script_text, result_varname, result));
    end if;
    return result;
  end;

end PYPL;
/
