/**
# LD_LIBRARY_PATH cannot be set once a process is running, so AFAIK it is not
# possible to set it here in Python code. Shame; the nice way would have been
# to just do this:
#
#  result := pypl.run_script(
#    'import sys, os' || LF ||
#    'os.environ["RDBASE"] = "/opt/rdkit"' || LF ||
#    'os.environ["LD_LIBRARY_PATH"] = "/opt/rdkit/lib"' || LF ||
#    'sys.path.append("/opt/rdkit")' || LF,
#    null
#  );
#
# I tried setting LD_LIBRARY_PATH in extproc.ora but it didn't seem to have an
# effect. So I have added it to the 'oracle' user's .bash_profile and since I
# was adding stuff there anyway, I added the following three lines:
#
#  export LD_LIBRARY_PATH=/opt/rdkit/lib
#  export RDBASE=/opt/rdkit
#  export PYTHONPATH=/opt/rdkit
#
# and restarted the database. rdkit can then be used directly by the 'oracle'
# user without additional configuration.
**/

set serveroutput on;

declare
  LF constant varchar2(1) := Chr(10);
  result pypl.pls_largest_varchar2;
begin
  -- Show that we can load and use rdkit.
  result := pypl.run_script(
    'from rdkit import Chem' || LF ||
    'result = Chem.MolToSmarts(Chem.MolFromSmiles("CCC"))' || LF,
    'result'
  );
  dbms_output.put_line('rdkit test: ' || result);
end;
/
